@@ -1,4 +1,4 @@
-# Copyright 2019 The Pigweed Authors
+# Copyright 2022 The Pigweed Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not
 # use this file except in compliance with the License. You may obtain a copy of
@@ -20,10 +20,13 @@ import logging
 import os
 import subprocess
 import sys
-from typing import List, Iterable, Optional
+import json
+from typing import Iterable, Optional
+from pathlib import Path
 
 import pw_cli.log
 
+from pw_build.python_runner import expand_expressions, GnPaths
 from pw_bloat.label import from_bloaty_tsv
 from pw_bloat.label_output import (BloatTableOutput, LineCharset, RstOutput,
                                    AsciiCharset)
@@ -35,58 +38,16 @@ MAX_COL_WIDTH = 50
 
 def parse_args() -> argparse.Namespace:
     """Parses the script's arguments."""
-    def delimited_list(delimiter: str, items: Optional[int] = None):
-        def _parser(arg: str):
-            args = arg.split(delimiter)
-
-            if items and len(args) != items:
-                raise argparse.ArgumentTypeError(
-                    'Argument must be a '
-                    f'{delimiter}-delimited list with {items} items: "{arg}"')
-
-            return args
-
-        return _parser
 
     parser = argparse.ArgumentParser(
         'Generate a size report card for binaries')
-    parser.add_argument('--bloaty-config',
-                        type=delimited_list(';'),
-                        required=True,
-                        help='Data source configuration for Bloaty')
-    parser.add_argument('--full',
-                        action='store_true',
-                        help='Display full bloat breakdown by symbol')
-    parser.add_argument('--data-sources',
-                        type=delimited_list(';'),
-                        help='List of sources to scan for')
-    parser.add_argument('--labels',
-                        type=delimited_list(';'),
-                        default='',
-                        help='Labels for output binaries')
-    parser.add_argument('--out-dir',
+    parser.add_argument('--gn-arg-path',
                         type=str,
                         required=True,
-                        help='Directory in which to write output files')
-    parser.add_argument('--target',
-                        type=str,
-                        required=True,
-                        help='Build target name')
-    parser.add_argument('--title',
-                        type=str,
-                        default='pw_bloat',
-                        help='Report title')
-    parser.add_argument('--source-filter',
-                        type=delimited_list(';'),
-                        help='Bloaty data source filter')
-    parser.add_argument('--single-target',
-                        type=str,
-                        help='Single executable target')
-    parser.add_argument('diff_targets',
-                        type=delimited_list(';', 2),
-                        nargs='*',
-                        metavar='DIFF_TARGET',
-                        help='Binary;base pairs to process')
+                        help='File path to json of binaries')
+    parser.add_argument('--single-report',
+                        action="store_true",
+                        help='Determine if calling single size report')
 
     return parser.parse_args()
 
@@ -144,12 +105,13 @@ def write_file(filename: str, contents: str, out_dir_file: str) -> None:
 
 
 def single_target_output(target: str, bloaty_config: str, target_out_file: str,
-                         out_dir: str, extra_args: Iterable[str]) -> int:
+                         out_dir: str, data_sources: Iterable[str],
+                         extra_args: Iterable[str]) -> int:
 
     try:
         single_output = run_bloaty(target,
                                    bloaty_config,
-                                   data_sources=['segment_names', 'symbols'],
+                                   data_sources=data_sources,
                                    extra_args=extra_args)
 
     except subprocess.CalledProcessError:
@@ -172,61 +134,91 @@ def single_target_output(target: str, bloaty_config: str, target_out_file: str,
     return 0
 
 
+# TODO(frolv) Copied from python_runner.py
+def _abspath(path: Path) -> Path:
+    """Turns a path into an absolute path, not resolving symlinks."""
+    return Path(os.path.abspath(path))
+
+
+def _translate_file_paths(gn_arg_dict: dict, single_report: bool) -> dict:
+    tool = gn_arg_dict['toolchain'] if gn_arg_dict['toolchain'] != gn_arg_dict[
+        'default_toolchain'] else ''
+    paths = GnPaths(root=_abspath(gn_arg_dict['root']),
+                    build=_abspath(Path.cwd()),
+                    toolchain=tool,
+                    cwd=_abspath(gn_arg_dict['cwd']))
+    for curr_arg in gn_arg_dict['binaries']:
+        curr_arg['target'] = list(expand_expressions(paths,
+                                                     curr_arg['target']))[0]
+        if not single_report:
+            curr_arg['base'] = list(expand_expressions(paths,
+                                                       curr_arg['base']))[0]
+    return gn_arg_dict
+
+
 def main() -> int:
     """Program entry point."""
 
     args = parse_args()
     extra_args = ['--tsv']
+    data_sources = ['segment_names', 'symbols']
+    gn_arg_dict = {}
+    json_file = open(args.gn_arg_path)
+    gn_arg_dict = json.load(json_file)
 
-    if args.single_target is not None:
-        if args.source_filter:
-            extra_args.extend(['--source-filter', args.source_filter])
-        return single_target_output(args.single_target, args.bloaty_config[0],
-                                    args.target, args.out_dir, extra_args)
+    gn_arg_dict = _translate_file_paths(gn_arg_dict, args.single_report)
 
-    base_binaries: List[str] = []
-    diff_binaries: List[str] = []
+    if args.single_report:
+        single_binary_args = gn_arg_dict['binaries'][0]
+        if single_binary_args['source_filter']:
+            extra_args.extend(
+                ['--source-filter', single_binary_args['source_filter']])
+        if single_binary_args['data_sources']:
+            data_sources = single_binary_args['data_sources']
 
-    try:
-        for binary, base in args.diff_targets:
-            diff_binaries.append(binary)
-            base_binaries.append(base)
+        return single_target_output(single_binary_args['target'],
+                                    single_binary_args['bloaty_config'],
+                                    gn_arg_dict['target_name'],
+                                    gn_arg_dict['out_dir'], data_sources,
+                                    extra_args)
 
-    except RuntimeError as err:
-        _LOG.error('%s: %s', sys.argv[0], err)
-        return 1
-
-    data_sources = args.data_sources
-
-    if args.data_sources is None:
-        data_sources = ['segment_names', 'symbols']
+    default_data_sources = ['segment_names', 'symbols']
 
     diff_report = ''
     rst_diff_report = ''
+    for curr_diff_binary in gn_arg_dict['binaries']:
+
+        curr_extra_args = extra_args.copy()
+        data_sources = default_data_sources
+
+        if curr_diff_binary['source_filter']:
+            curr_extra_args.extend(
+                ['--source-filter', curr_diff_binary['source_filter']])
+
+        if curr_diff_binary['data_sources']:
+            data_sources = curr_diff_binary['data_sources']
 
-    for i, binary in enumerate(diff_binaries):
-        if args.source_filter is not None and args.source_filter[i]:
-            extra_args.extend(['--source-filter', args.source_filter[i]])
         try:
-            single_output_base = run_bloaty(base_binaries[i],
-                                            args.bloaty_config[i],
+            single_output_base = run_bloaty(curr_diff_binary["base"],
+                                            curr_diff_binary['bloaty_config'],
                                             data_sources=data_sources,
-                                            extra_args=extra_args)
+                                            extra_args=curr_extra_args)
 
         except subprocess.CalledProcessError:
             _LOG.error('%s: failed to run base size report on %s', sys.argv[0],
-                       base_binaries[i])
+                       curr_diff_binary["base"])
             return 1
 
         try:
-            single_output_target = run_bloaty(binary,
-                                              args.bloaty_config[i],
-                                              data_sources=data_sources,
-                                              extra_args=extra_args)
+            single_output_target = run_bloaty(
+                curr_diff_binary["target"],
+                curr_diff_binary['bloaty_config'],
+                data_sources=data_sources,
+                extra_args=curr_extra_args)
 
         except subprocess.CalledProcessError:
             _LOG.error('%s: failed to run target size report on %s',
-                       sys.argv[0], binary)
+                       sys.argv[0], curr_diff_binary["target"])
             return 1
 
         if not single_output_target or not single_output_base:
@@ -237,19 +229,22 @@ def main() -> int:
             single_output_target.decode().splitlines())
         diff_dsm = target_dsm.diff(base_dsm)
 
-        diff_report += BloatTableOutput(diff_dsm, MAX_COL_WIDTH,
-                                        LineCharset).create_table()
-
-        print(diff_report)
-        curr_rst_report = RstOutput(diff_dsm, args.labels[i])
-        if i == 0:
+        diff_report += BloatTableOutput(
+            diff_dsm,
+            MAX_COL_WIDTH,
+            LineCharset,
+            diff_label=curr_diff_binary['label']).create_table()
+        curr_rst_report = RstOutput(diff_dsm, curr_diff_binary['label'])
+        if rst_diff_report == '':
             rst_diff_report = curr_rst_report.create_table()
         else:
             rst_diff_report += f"{curr_rst_report.add_report_row()}\n"
-        extra_args = ['--tsv']
 
-    write_file(args.target, rst_diff_report, args.out_dir)
-    write_file(f'{args.target}.txt', diff_report, args.out_dir)
+    print(diff_report)
+    write_file(gn_arg_dict['target_name'], rst_diff_report,
+               gn_arg_dict['out_dir'])
+    write_file(f"{gn_arg_dict['target_name']}.txt", diff_report,
+               gn_arg_dict['out_dir'])
 
     return 0
 
