@@ -92,10 +92,7 @@ def check_version(manifest, cipd_path, package_name):
         instance_id = json.load(ins)['instance_id']
 
     with open(manifest, 'r') as ins:
-        data = json.load(ins)
-    # TODO(pwbug/599) Always assume this is a dict.
-    if isinstance(data, dict):
-        data = data['packages']
+        data = json.load(ins)['packages']
 
     path = None
     expected_version = None
