@@ -21,6 +21,7 @@ import logging
 import os
 from pathlib import Path
 import sys
+import time
 from threading import Thread
 from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 
@@ -66,18 +67,16 @@ from pw_console.python_logging import all_loggers
 from pw_console.quit_dialog import QuitDialog
 from pw_console.repl_pane import ReplPane
 import pw_console.style
+from pw_console.test_mode import start_fake_logger
 import pw_console.widgets.checkbox
 from pw_console.widgets import FloatingWindowPane
 import pw_console.widgets.mouse_handlers
 from pw_console.window_manager import WindowManager
 
 _LOG = logging.getLogger(__package__)
+_ROOT_LOG = logging.getLogger('')
 
-# Fake logger for --test-mode
-FAKE_DEVICE_LOGGER_NAME = 'pw_console_fake_device'
-_FAKE_DEVICE_LOG = logging.getLogger(FAKE_DEVICE_LOGGER_NAME)
-# Don't send fake_device logs to the root Python logger.
-_FAKE_DEVICE_LOG.propagate = False
+_SYSTEM_COMMAND_LOG = logging.getLogger('pw_console_system_command')
 
 MAX_FPS = 30
 MIN_REDRAW_INTERVAL = (60.0 / MAX_FPS) / 60.0
@@ -143,6 +142,11 @@ class ConsoleApp:
         self.prefs = prefs if prefs else ConsolePrefs()
         self.color_depth = get_default_colordepth(color_depth)
 
+        # Max frequency in seconds of prompt_toolkit UI redraws triggered by new
+        # log lines.
+        self.log_ui_update_frequency = 0.1  # 10 FPS
+        self._last_ui_update_time = time.time()
+
         # Create a default global and local symbol table. Values are the same
         # structure as what is returned by globals():
         #   https://docs.python.org/3/library/functions.html#globals
@@ -183,6 +187,7 @@ class ConsoleApp:
 
         # Event loop for executing user repl code.
         self.user_code_loop = asyncio.new_event_loop()
+        self.test_mode_log_loop = asyncio.new_event_loop()
 
         self.app_title = app_title if app_title else 'Pigweed Console'
 
@@ -247,6 +252,8 @@ class ConsoleApp:
         )
         self.pw_ptpython_repl.use_code_colorscheme(self.prefs.code_theme)
 
+        self.system_command_output_pane: Optional[LogPane] = None
+
         if self.prefs.swap_light_and_dark:
             self.toggle_light_theme()
 
@@ -869,6 +876,11 @@ class ConsoleApp:
                         daemon=True)
         thread.start()
 
+    def _test_mode_log_thread_entry(self):
+        """Entry point for the user code thread."""
+        asyncio.set_event_loop(self.test_mode_log_loop)
+        self.test_mode_log_loop.run_forever()
+
     def _update_help_window(self):
         """Generate the help window text based on active pane keybindings."""
         # Add global mouse bindings to the help text.
@@ -937,6 +949,16 @@ class ConsoleApp:
         """Quit the console prompt_toolkit application UI."""
         self.application.exit()
 
+    def logs_redraw(self):
+        emit_time = time.time()
+        # Has enough time passed since last UI redraw due to new logs?
+        if emit_time > self._last_ui_update_time + self.log_ui_update_frequency:
+            # Update last log time
+            self._last_ui_update_time = emit_time
+
+            # Trigger Prompt Toolkit UI redraw.
+            self.redraw_ui()
+
     def redraw_ui(self):
         """Redraw the prompt_toolkit UI."""
         if hasattr(self, 'application'):
@@ -944,10 +966,31 @@ class ConsoleApp:
             # loop.
             self.application.invalidate()
 
+    def setup_command_runner_log_pane(self) -> None:
+        if not self.system_command_output_pane is None:
+            return
+
+        self.system_command_output_pane = LogPane(application=self,
+                                                  pane_title='Shell Output')
+        self.system_command_output_pane.add_log_handler(_SYSTEM_COMMAND_LOG,
+                                                        level_name='INFO')
+        self.system_command_output_pane.log_view.log_store.formatter = (
+            logging.Formatter('%(message)s'))
+        self.system_command_output_pane.table_view = False
+        self.system_command_output_pane.show_pane = True
+        # Enable line wrapping
+        self.system_command_output_pane.toggle_wrap_lines()
+        # Blank right side toolbar text
+        self.system_command_output_pane._pane_subtitle = ' '  # pylint: disable=protected-access
+        self.window_manager.add_pane(self.system_command_output_pane)
+
     async def run(self, test_mode=False):
         """Start the prompt_toolkit UI."""
         if test_mode:
-            background_log_task = asyncio.create_task(self.log_forever())
+            background_log_task = start_fake_logger(
+                lines=self.user_guide_window.help_text_area.document.lines,
+                log_thread_entry=self._test_mode_log_thread_entry,
+                log_thread_loop=self.test_mode_log_loop)
 
         # Repl pane has focus by default, if it's hidden switch focus to another
         # visible pane.
@@ -961,44 +1004,6 @@ class ConsoleApp:
             if test_mode:
                 background_log_task.cancel()
 
-    async def log_forever(self):
-        """Test mode async log generator coroutine that runs forever."""
-        message_count = 0
-        # Sample log line format:
-        # Log message [=         ] # 100
-
-        # Fake module column names.
-        module_names = ['APP', 'RADIO', 'BAT', 'USB', 'CPU']
-        while True:
-            if message_count > 32 or message_count < 2:
-                await asyncio.sleep(1)
-            bar_size = 10
-            position = message_count % bar_size
-            bar_content = " " * (bar_size - position - 1) + "="
-            if position > 0:
-                bar_content = "=".rjust(position) + " " * (bar_size - position)
-            new_log_line = 'Log message [{}] # {}'.format(
-                bar_content, message_count)
-            if message_count % 10 == 0:
-                new_log_line += (
-                    ' Lorem ipsum \033[34m\033[1mdolor sit amet\033[0m'
-                    ', consectetur '
-                    'adipiscing elit.') * 8
-            if message_count % 11 == 0:
-                new_log_line += ' '
-                new_log_line += (
-                    '[PYTHON] START\n'
-                    'In []: import time;\n'
-                    '        def t(s):\n'
-                    '            time.sleep(s)\n'
-                    '            return "t({}) seconds done".format(s)\n\n')
-
-            module_name = module_names[message_count % len(module_names)]
-            _FAKE_DEVICE_LOG.info(new_log_line,
-                                  extra=dict(extra_metadata_fields=dict(
-                                      module=module_name, file='fake_app.cc')))
-            message_count += 1
-
 
 # TODO(tonymd): Remove this alias when not used by downstream projects.
 def embed(
