@@ -16,7 +16,6 @@
 from __future__ import annotations
 import collections
 import logging
-import sys
 from datetime import datetime
 from typing import Dict, List, Optional, TYPE_CHECKING
 
@@ -91,9 +90,6 @@ class LogStore(logging.Handler):
         # and end of the iterable.
         self.logs: collections.deque = collections.deque()
 
-        # Estimate of the logs in memory.
-        self.byte_size: int = 0
-
         # Only allow this many log lines in memory.
         self.max_history_size: int = 1000000
 
@@ -146,7 +142,6 @@ class LogStore(logging.Handler):
     def clear_logs(self):
         """Erase all stored pane lines."""
         self.logs = collections.deque()
-        self.byte_size = 0
         self.channel_counts = {}
         self.channel_formatted_prefix_widths = {}
         self.line_index = 0
@@ -227,12 +222,6 @@ class LogStore(logging.Handler):
         # Check for bigger column widths.
         self.table.update_metadata_column_widths(self.logs[-1])
 
-        # Update estimated byte_size.
-        self.byte_size += sys.getsizeof(self.logs[-1])
-        # If the total log lines is > max_history_size, delete the oldest line.
-        if self.get_total_count() > self.max_history_size:
-            self.byte_size -= sys.getsizeof(self.logs.popleft())
-
     def emit(self, record) -> None:
         """Process a new log record.
 
