@@ -21,10 +21,13 @@ from pw_package.packages import boringssl  # pylint: disable=unused-import
 from pw_package.packages import chromium_verifier  # pylint: disable=unused-import
 from pw_package.packages import crlset  # pylint: disable=unused-import
 from pw_package.packages import freertos  # pylint: disable=unused-import
+from pw_package.packages import googletest  # pylint: disable=unused-import
 from pw_package.packages import mbedtls  # pylint: disable=unused-import
 from pw_package.packages import micro_ecc  # pylint: disable=unused-import
 from pw_package.packages import nanopb
+from pw_package.packages import pico_sdk  # pylint: disable=unused-import
 from pw_package.packages import protobuf  # pylint: disable=unused-import
+from pw_package.packages import smartfusion_mss  # pylint: disable=unused-import
 from pw_package.packages import stm32cube  # pylint: disable=unused-import
 
 
