@@ -18,7 +18,6 @@ from pw_presubmit import (
     Check,
     format_code,
     PresubmitContext,
-    PresubmitFailure,
     filter_paths,
 )
 
@@ -33,38 +32,40 @@ def pragma_once(ctx: PresubmitContext) -> None:
                 if line.startswith('#pragma once'):
                     break
             else:
-                raise PresubmitFailure('#pragma once is missing!', path=path)
+                ctx.fail('#pragma once is missing!', path=path)
 
 
 @Check
 def asan(ctx: PresubmitContext) -> None:
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, 'asan')
+    build.gn_gen(ctx)
+    build.ninja(ctx, 'asan')
 
 
 @Check
 def msan(ctx: PresubmitContext) -> None:
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, 'msan')
+    build.gn_gen(ctx)
+    build.ninja(ctx, 'msan')
 
 
 @Check
 def tsan(ctx: PresubmitContext) -> None:
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, 'tsan')
+    build.gn_gen(ctx)
+    build.ninja(ctx, 'tsan')
 
 
 @Check
 def ubsan(ctx: PresubmitContext) -> None:
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, 'ubsan')
+    build.gn_gen(ctx)
+    build.ninja(ctx, 'ubsan')
 
 
 @Check
 def runtime_sanitizers(ctx: PresubmitContext) -> None:
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, 'runtime_sanitizers')
+    build.gn_gen(ctx)
+    build.ninja(ctx, 'runtime_sanitizers')
 
 
 def all_sanitizers():
-    return [asan, msan, tsan, ubsan, runtime_sanitizers]
+    # TODO(b/234876100): msan will not work until the C++ standard library
+    # included in the sysroot has a variant built with msan.
+    return [asan, tsan, ubsan, runtime_sanitizers]
