@@ -76,7 +76,31 @@ def _at_all_optimization_levels(target):
 # Build presubmit checks
 #
 def gn_clang_build(ctx: PresubmitContext):
-    build_targets = list(_at_all_optimization_levels('host_clang'))
+    """Checks all compile targets that rely on LLVM tooling."""
+    build_targets = [
+        *_at_all_optimization_levels('host_clang'),
+        'cpp14_compatibility',
+        'cpp20_compatibility',
+        'asan',
+        'tsan',
+        'ubsan',
+        'runtime_sanitizers',
+
+        # TODO(b/234876100): msan will not work until the C++ standard library
+        # included in the sysroot has a variant built with msan.
+    ]
+
+    # clang-tidy doesn't run on Windows.
+    if sys.platform != 'win32':
+        build_targets.append('static_analysis')
+
+    # QEMU doesn't run on Windows.
+    if sys.platform != 'win32':
+        # TODO(b/244604080): For the pw::InlineString tests, qemu_clang_debug
+        #     and qemu_clang_speed_optimized produce a binary too large for the
+        #     QEMU target's 256KB flash. Restore debug and speed optimized
+        #     builds when this is fixed.
+        build_targets.append('qemu_clang_size_optimized')
 
     # TODO(b/240982565): SocketStream currently requires Linux.
     if sys.platform.startswith('linux'):
@@ -143,6 +167,7 @@ def gn_full_qemu_check(ctx: PresubmitContext):
 
 @_BUILD_FILE_FILTER.apply_to_check()
 def gn_combined_build_check(ctx: PresubmitContext) -> None:
+    """Run most host and device (QEMU) tests."""
     build_targets = [
         *_at_all_optimization_levels('stm32f429i'),
         *_at_all_optimization_levels(f'host_{_HOST_COMPILER}'),
@@ -158,9 +183,19 @@ def gn_combined_build_check(ctx: PresubmitContext) -> None:
         build_targets.append('cpp14_compatibility')
         build_targets.append('cpp20_compatibility')
 
+    # clang-tidy doesn't run on Windows.
+    if sys.platform != 'win32':
+        build_targets.append('static_analysis')
+
+    # QEMU doesn't run on Windows.
     if sys.platform != 'win32':
         build_targets.extend(_at_all_optimization_levels('qemu_gcc'))
-        build_targets.extend(_at_all_optimization_levels('qemu_clang'))
+
+        # TODO(b/244604080): For the pw::InlineString tests, qemu_clang_debug
+        #     and qemu_clang_speed_optimized produce a binary too large for the
+        #     QEMU target's 256KB flash. Restore debug and speed optimized
+        #     builds when this is fixed.
+        build_targets.append('qemu_clang_size_optimized')
 
     # TODO(b/240982565): SocketStream currently requires Linux.
     if sys.platform.startswith('linux'):
@@ -385,6 +420,8 @@ _TARGETS_THAT_DO_NOT_BUILD_WITH_BAZEL = (
     '-//pw_blob_store/...:all',
     '-//pw_boot/...:all',
     '-//pw_cpu_exception_cortex_m/...:all',
+    '-//pw_chrono:chrono_proto_pb2',
+    '-//pw_chrono/py/...:all',
     '-//pw_crypto/...:all',  # TODO(b/236321905) Remove when passing.
     '-//pw_file/...:all',
     '-//pw_function:function_test',  # TODO(b/241821115) Remove when passing.
@@ -420,6 +457,8 @@ _TARGETS_THAT_DO_NOT_BUILD_WITH_BAZEL = (
     '-//pw_thread/py/...:all',
     '-//pw_thread:thread_proto_py_pb2',
     '-//pw_thread:thread_proto_py_pb2_genproto',
+    '-//pw_thread:thread_snapshot_service_py_pb2',
+    '-//pw_thread:thread_snapshot_service_py_pb2_genproto',
     '-//pw_thread_embos/...:all',
     '-//pw_thread_freertos/...:all',
     '-//pw_thread_threadx/...:all',
@@ -548,6 +587,7 @@ _EXCLUDE_FROM_COPYRIGHT_NOTICE: Sequence[str] = (
     r'\.csv$',
     r'\.elf$',
     r'\.gif$',
+    r'\.ico$',
     r'\.jpg$',
     r'\.json$',
     r'\.png$',
