@@ -170,6 +170,45 @@ class Programs(collections.abc.Mapping):
         return len(self._programs)
 
 
+@dataclasses.dataclass
+class LuciContext:
+    """LUCI-specific information about the environment."""
+    buildbucket_id: int
+    build_number: int
+    project: str
+    bucket: str
+    builder: str
+    swarming_task_id: str
+
+    @staticmethod
+    def create_from_environment():
+        luci_vars = [
+            'BUILDBUCKET_ID',
+            'BUILDBUCKET_NAME',
+            'BUILD_NUMBER',
+            'SWARMING_TASK_ID',
+        ]
+        if any(x for x in luci_vars if x not in os.environ):
+            return None
+
+        project, bucket, builder = os.environ['BUILDBUCKET_NAME'].split(':')
+
+        bbid: int = 0
+        try:
+            bbid = int(os.environ['BUILDBUCKET_ID'])
+        except ValueError:
+            pass
+
+        return LuciContext(
+            buildbucket_id=bbid,
+            build_number=int(os.environ['BUILD_NUMBER']),
+            project=project,
+            bucket=bucket,
+            builder=builder,
+            swarming_task_id=os.environ['SWARMING_TASK_ID'],
+        )
+
+
 @dataclasses.dataclass
 class PresubmitContext:
     """Context passed into presubmit checks."""
@@ -178,9 +217,10 @@ class PresubmitContext:
     output_dir: Path
     paths: Tuple[Path, ...]
     package_root: Path
-    # TODO(b/233808334): Remove default value after updating downstream
-    # projects to use this new value.
-    override_gn_args: Dict[str, str] = dataclasses.field(default_factory=dict)
+    luci: Optional[LuciContext]
+    override_gn_args: Dict[str, str]
+    num_jobs: Optional[int] = None
+    continue_after_build_error: bool = False
     _failed: bool = False
 
     @property
@@ -273,7 +313,8 @@ class Presubmit:
     """Runs a series of presubmit checks on a list of files."""
     def __init__(self, root: Path, repos: Sequence[Path],
                  output_directory: Path, paths: Sequence[Path],
-                 package_root: Path, override_gn_args: Dict[str, str]):
+                 package_root: Path, override_gn_args: Dict[str, str],
+                 continue_after_build_error: bool):
         self._root = root.resolve()
         self._repos = tuple(repos)
         self._output_directory = output_directory.resolve()
@@ -282,6 +323,7 @@ class Presubmit:
             tools.relative_paths(self._paths, self._root))
         self._package_root = package_root.resolve()
         self._override_gn_args = override_gn_args
+        self._continue_after_build_error = continue_after_build_error
 
     def run(self, program: Program, keep_going: bool = False) -> bool:
         """Executes a series of presubmit checks on the paths."""
@@ -368,6 +410,11 @@ class Presubmit:
                 f'{total} checks on {plural(self._paths, "file")}: {summary}',
                 _format_time(time_s)))
 
+    def _create_presubmit_context(  # pylint: disable=no-self-use
+            self, **kwargs):
+        """Create a PresubmitContext. Override if needed in subclasses."""
+        return PresubmitContext(**kwargs)
+
     @contextlib.contextmanager
     def _context(self, name: str, paths: Tuple[Path, ...]):
         # There are many characters banned from filenames on Windows. To
@@ -384,13 +431,15 @@ class Presubmit:
         try:
             _LOG.addHandler(handler)
 
-            yield PresubmitContext(
+            yield self._create_presubmit_context(
                 root=self._root,
                 repos=self._repos,
                 output_dir=output_directory,
                 paths=paths,
                 package_root=self._package_root,
                 override_gn_args=self._override_gn_args,
+                continue_after_build_error=self._continue_after_build_error,
+                luci=LuciContext.create_from_environment(),
             )
 
         finally:
@@ -459,7 +508,9 @@ def run(  # pylint: disable=too-many-arguments
         package_root: Path = None,
         only_list_steps: bool = False,
         override_gn_args: Sequence[Tuple[str, str]] = (),
-        keep_going: bool = False) -> bool:
+        keep_going: bool = False,
+        continue_after_build_error: bool = False,
+        presubmit_class: type = Presubmit) -> bool:
     """Lists files in the current Git repo and runs a Presubmit with them.
 
     This changes the directory to the root of the Git repository after listing
@@ -484,7 +535,10 @@ def run(  # pylint: disable=too-many-arguments
         package_root: where to place package files
         only_list_steps: print step names instead of running them
         override_gn_args: additional GN args to set on steps
-        keep_going: whether to continue running checks if an error occurs
+        keep_going: continue running presubmit steps after a step fails
+        continue_after_build_error: continue building if a build step fails
+        presubmit_class: class to use to run Presubmits, should inherit from
+            Presubmit class above
 
     Returns:
         True if all presubmit checks succeeded
@@ -520,13 +574,14 @@ def run(  # pylint: disable=too-many-arguments
     if package_root is None:
         package_root = output_directory / 'packages'
 
-    presubmit = Presubmit(
+    presubmit = presubmit_class(
         root=root,
         repos=repos,
         output_directory=output_directory,
         paths=files,
         package_root=package_root,
         override_gn_args=dict(override_gn_args or {}),
+        continue_after_build_error=continue_after_build_error,
     )
 
     if only_list_steps:
