@@ -37,6 +37,7 @@ PROTO_H_EXTENSION = '.pwpb.h'
 PROTO_CC_EXTENSION = '.pwpb.cc'
 
 PROTOBUF_NAMESPACE = '::pw::protobuf'
+_INTERNAL_NAMESPACE = '::pw::protobuf::internal'
 
 
 class ClassType(enum.Enum):
@@ -348,6 +349,19 @@ class MessageProperty(ProtoMember):
         """
         raise NotImplementedError()
 
+    def is_string(self) -> bool:  # pylint: disable=no-self-use
+        """True if this field is a string field (as opposed to bytes)."""
+        return False
+
+    @staticmethod
+    def repeated_field_container(type_name: str, max_size: int) -> str:
+        """Returns the container type used for repeated fields.
+
+        Defaults to ::pw::Vector<type, max_size>. String fields use
+        ::pw::InlineString<max_size> instead.
+        """
+        return f'::pw::Vector<{type_name}, {max_size}>'
+
     def use_callback(self) -> bool:  # pylint: disable=no-self-use
         """Returns whether the decoder should use a callback."""
         options = self._field.options()
@@ -360,6 +374,9 @@ class MessageProperty(ProtoMember):
         return (self._field.is_optional() and self.max_size() == 0
                 and self.wire_type() != 'kDelimited')
 
+    def is_repeated(self) -> bool:
+        return self._field.is_repeated()
+
     def max_size(self) -> int:
         """Returns the maximum size of the field."""
         if self._field.is_repeated():
@@ -369,7 +386,7 @@ class MessageProperty(ProtoMember):
 
         return 0
 
-    def fixed_size(self) -> bool:
+    def is_fixed_size(self) -> bool:
         """Returns whether the decoder should use a fixed sized field."""
         if self._field.is_repeated():
             options = self._field.options()
@@ -398,20 +415,20 @@ class MessageProperty(ProtoMember):
             return (self.type_name(from_root), self.name())
 
         # Fixed size fields use std::array.
-        if self.fixed_size():
+        if self.is_fixed_size():
             return ('std::array<{}, {}>'.format(self.type_name(from_root),
                                                 max_size), self.name())
 
         # Otherwise prefer pw::Vector for repeated fields.
-        return ('::pw::Vector<{}, {}>'.format(self.type_name(from_root),
+        return (self.repeated_field_container(self.type_name(from_root),
                                               max_size), self.name())
 
     def _varint_type_table_entry(self) -> str:
         if self.wire_type() == 'kVarint':
-            return '{}::VarintType::{}'.format(PROTOBUF_NAMESPACE,
+            return '{}::VarintType::{}'.format(_INTERNAL_NAMESPACE,
                                                self.varint_decode_type())
 
-        return f'static_cast<{PROTOBUF_NAMESPACE}::VarintType>(0)'
+        return f'static_cast<{_INTERNAL_NAMESPACE}::VarintType>(0)'
 
     def _wire_type_table_entry(self) -> str:
         return '{}::WireType::{}'.format(PROTOBUF_NAMESPACE, self.wire_type())
@@ -419,6 +436,10 @@ class MessageProperty(ProtoMember):
     def _elem_size_table_entry(self) -> str:
         return 'sizeof({})'.format(self.type_name())
 
+    def _bool_attr(self, attr: str) -> str:
+        """C++ string for a bool argument that includes the argument name."""
+        return f'/*{attr}=*/{bool(getattr(self, attr)())}'.lower()
+
     def table_entry(self) -> List[str]:
         """Table entry."""
         return [
@@ -426,10 +447,11 @@ class MessageProperty(ProtoMember):
             self._wire_type_table_entry(),
             self._elem_size_table_entry(),
             self._varint_type_table_entry(),
-            'true' if self.fixed_size() else 'false',
-            'true' if self._field.is_repeated() else 'false',
-            'true' if self.is_optional() else 'false',
-            'true' if self.use_callback() else 'false',
+            self._bool_attr('is_string'),
+            self._bool_attr('is_fixed_size'),
+            self._bool_attr('is_repeated'),
+            self._bool_attr('is_optional'),
+            self._bool_attr('use_callback'),
             'offsetof(Message, {})'.format(self.name()),
             'sizeof(Message::{})'.format(self.name()),
             self.sub_table(),
@@ -1492,7 +1514,7 @@ class BytesProperty(MessageProperty):
 
         return 0
 
-    def fixed_size(self) -> bool:
+    def is_fixed_size(self) -> bool:
         if not self._field.is_repeated():
             options = self._field.options()
             assert options is not None
@@ -1561,12 +1583,19 @@ class StringProperty(MessageProperty):
 
         return 0
 
-    def fixed_size(self) -> bool:
+    def is_fixed_size(self) -> bool:
         return False
 
     def wire_type(self) -> str:
         return 'kDelimited'
 
+    def is_string(self) -> bool:
+        return True
+
+    @staticmethod
+    def repeated_field_container(type_name: str, max_size: int) -> str:
+        return f'::pw::InlineBasicString<{type_name}, {max_size}>'
+
     def _size_fn(self) -> str:
         # This uses the WithoutValue method to ensure that the maximum length
         # of the delimited field size varint is used. This accounts for scratch
@@ -2110,7 +2139,7 @@ def generate_table_for_message(message: ProtoMessage, root: ProtoNode,
                     prop.name()))
         output.write_line('static_assert(sizeof(Message::{}) <= '
                           '{}::MessageField::kMaxFieldSize);'.format(
-                              prop.name(), PROTOBUF_NAMESPACE))
+                              prop.name(), _INTERNAL_NAMESPACE))
 
     # Zero-length C arrays are not permitted by the C++ standard, so only
     # generate the message fields array if it is non-empty. Zero-length
@@ -2121,7 +2150,7 @@ def generate_table_for_message(message: ProtoMessage, root: ProtoNode,
     # not. Only the span is referenced elsewhere.
     if properties:
         output.write_line(
-            f'inline constexpr {PROTOBUF_NAMESPACE}::MessageField '
+            f'inline constexpr {_INTERNAL_NAMESPACE}::MessageField '
             ' _kMessageFields[] = {')
 
         # Generate members for each of the message's fields.
@@ -2134,11 +2163,11 @@ def generate_table_for_message(message: ProtoMessage, root: ProtoNode,
         output.write_line('PW_MODIFY_DIAGNOSTICS_POP();')
 
         output.write_line(
-            f'inline constexpr pw::span<const {PROTOBUF_NAMESPACE}::'
+            f'inline constexpr pw::span<const {_INTERNAL_NAMESPACE}::'
             'MessageField> kMessageFields = _kMessageFields;')
     else:
         output.write_line(
-            f'inline constexpr pw::span<const {PROTOBUF_NAMESPACE}::'
+            f'inline constexpr pw::span<const {_INTERNAL_NAMESPACE}::'
             'MessageField> kMessageFields;')
 
     output.write_line(f'}}  // namespace {namespace}')
@@ -2249,6 +2278,7 @@ def generate_code_for_package(file_descriptor_proto, package: ProtoNode,
     output.write_line('#include "pw_span/span.h"')
     output.write_line('#include "pw_status/status.h"')
     output.write_line('#include "pw_status/status_with_size.h"')
+    output.write_line('#include "pw_string/string.h"')
 
     for imported_file in file_descriptor_proto.dependency:
         generated_header = _proto_filename_to_generated_header(imported_file)
