@@ -13,6 +13,7 @@
 # the License.
 """Builds and manages databases of tokenized strings."""
 
+from abc import abstractmethod
 import collections
 import csv
 from dataclasses import dataclass
@@ -22,15 +23,18 @@ import logging
 from pathlib import Path
 import re
 import struct
+import subprocess
 from typing import (BinaryIO, Callable, Dict, Iterable, Iterator, List,
-                    NamedTuple, Optional, Pattern, Tuple, Union, ValuesView)
+                    NamedTuple, Optional, Pattern, TextIO, Tuple, Union,
+                    ValuesView)
+from uuid import uuid4
 
 DATE_FORMAT = '%Y-%m-%d'
 DEFAULT_DOMAIN = ''
 
-# The default hash length to use. This value only applies when hashing strings
-# from a legacy-style ELF with plain strings. New tokenized string entries
-# include the token alongside the string.
+# The default hash length to use for C-style hashes. This value only applies
+# when manually hashing strings to recreate token calculations in C. The C++
+# hash function does not have a maximum length.
 #
 # This MUST match the default value of PW_TOKENIZER_CFG_C_HASH_LENGTH in
 # pw_tokenizer/public/pw_tokenizer/config.h.
@@ -46,11 +50,13 @@ def _value(char: Union[int, str]) -> int:
 
 
 def pw_tokenizer_65599_hash(string: Union[str, bytes],
+                            *,
                             hash_length: int = None) -> int:
-    """Hashes the provided string.
+    """Hashes the string with the hash function used to generate tokens in C++.
 
-    This hash function is only used when adding tokens from legacy-style
-    tokenized strings in an ELF, which do not include the token.
+    This hash function is used calculate tokens from strings in Python. It is
+    not used when extracting tokens from an ELF, since the token is stored in
+    the ELF as part of tokenization.
     """
     hash_value = len(string)
     coefficient = TOKENIZER_HASH_CONSTANT
@@ -62,8 +68,10 @@ def pw_tokenizer_65599_hash(string: Union[str, bytes],
     return hash_value
 
 
-def default_hash(string: Union[str, bytes]) -> int:
-    return pw_tokenizer_65599_hash(string, DEFAULT_C_HASH_LENGTH)
+def c_hash(string: Union[str, bytes],
+           hash_length: int = DEFAULT_C_HASH_LENGTH) -> int:
+    """Hashes the string with the hash function used in C."""
+    return pw_tokenizer_65599_hash(string, hash_length=hash_length)
 
 
 class _EntryKey(NamedTuple):
@@ -116,20 +124,20 @@ class Database:
     def __init__(self, entries: Iterable[TokenizedStringEntry] = ()):
         """Creates a token database."""
         # The database dict stores each unique (token, string) entry.
-        self._database: Dict[_EntryKey, TokenizedStringEntry] = {
-            entry.key(): entry
-            for entry in entries
-        }
+        self._database: Dict[_EntryKey, TokenizedStringEntry] = {}
 
         # This is a cache for fast token lookup that is built as needed.
         self._cache: Optional[Dict[int, List[TokenizedStringEntry]]] = None
 
+        self.add(entries)
+
     @classmethod
     def from_strings(
-            cls,
-            strings: Iterable[str],
-            domain: str = DEFAULT_DOMAIN,
-            tokenize: Callable[[str], int] = default_hash) -> 'Database':
+        cls,
+        strings: Iterable[str],
+        domain: str = DEFAULT_DOMAIN,
+        tokenize: Callable[[str],
+                           int] = pw_tokenizer_65599_hash) -> 'Database':
         """Creates a Database from an iterable of strings."""
         return cls((TokenizedStringEntry(tokenize(string), string, domain)
                     for string in strings))
@@ -201,7 +209,10 @@ class Database:
         return removed
 
     def add(self, entries: Iterable[TokenizedStringEntry]) -> None:
-        """Adds new entries and updates date_removed for existing entries."""
+        """Adds new entries and updates date_removed for existing entries.
+
+        If the added tokens have removal dates, the newest date is used.
+        """
         self._cache = None
 
         for new_entry in entries:
@@ -209,10 +220,17 @@ class Database:
             try:
                 entry = self._database[new_entry.key()]
                 entry.domain = new_entry.domain
-                entry.date_removed = None
+
+                # Keep the latest removal date between the two entries.
+                if new_entry.date_removed is None:
+                    entry.date_removed = None
+                elif (entry.date_removed
+                      and entry.date_removed < new_entry.date_removed):
+                    entry.date_removed = new_entry.date_removed
             except KeyError:
+                # Make a copy to avoid unintentially updating the database.
                 self._database[new_entry.key()] = TokenizedStringEntry(
-                    new_entry.token, new_entry.string, new_entry.domain)
+                    **vars(new_entry))
 
     def purge(
         self,
@@ -284,10 +302,19 @@ class Database:
             for value in self._database.values():
                 value.string = search.sub(replacement, value.string)
 
+    def difference(self, other: 'Database') -> 'Database':
+        """Returns a new Database with entries in this DB not in the other."""
+        return Database(e for k, e in self._database.items()
+                        if k not in other._database)  # pylint: disable=protected-access
+
     def __len__(self) -> int:
         """Returns the number of entries in the database."""
         return len(self.entries())
 
+    def __bool__(self) -> bool:
+        """True if the database is non-empty."""
+        return bool(self._database)
+
     def __str__(self) -> str:
         """Outputs the database as CSV."""
         csv_output = io.BytesIO()
@@ -295,7 +322,7 @@ class Database:
         return csv_output.getvalue().decode()
 
 
-def parse_csv(fd) -> Iterable[TokenizedStringEntry]:
+def parse_csv(fd: TextIO) -> Iterable[TokenizedStringEntry]:
     """Parses TokenizedStringEntries from a CSV token database file."""
     for line in csv.reader(fd):
         try:
@@ -315,12 +342,17 @@ def parse_csv(fd) -> Iterable[TokenizedStringEntry]:
 def write_csv(database: Database, fd: BinaryIO) -> None:
     """Writes the database as CSV to the provided binary file."""
     for entry in sorted(database.entries()):
-        # Align the CSV output to 10-character columns for improved readability.
-        # Use \n instead of RFC 4180's \r\n.
-        fd.write('{:08x},{:10},"{}"\n'.format(
-            entry.token,
-            entry.date_removed.strftime(DATE_FORMAT) if entry.date_removed else
-            '', entry.string.replace('"', '""')).encode())  # escape " as ""
+        _write_csv_line(fd, entry)
+
+
+def _write_csv_line(fd: BinaryIO, entry: TokenizedStringEntry):
+    """Write a line in CSV format to the provided binary file."""
+    # Align the CSV output to 10-character columns for improved readability.
+    # Use \n instead of RFC 4180's \r\n.
+    fd.write('{:08x},{:10},"{}"\n'.format(
+        entry.token,
+        entry.date_removed.strftime(DATE_FORMAT) if entry.date_removed else '',
+        entry.string.replace('"', '""')).encode())  # escape " as ""
 
 
 class _BinaryFileFormat(NamedTuple):
@@ -445,23 +477,203 @@ class DatabaseFile(Database):
     This class adds the write_to_file() method that writes to file from which it
     was created in the correct format (CSV or binary).
     """
-    def __init__(self, path: Union[Path, str]):
-        self.path = Path(path)
+    def __init__(self, path: Path,
+                 entries: Iterable[TokenizedStringEntry]) -> None:
+        super().__init__(entries)
+        self.path = path
+
+    @staticmethod
+    def load(path: Path) -> 'DatabaseFile':
+        """Creates a DatabaseFile that coincides to the file type."""
+        if path.is_dir():
+            return _DirectoryDatabase(path)
 
         # Read the path as a packed binary file.
-        with self.path.open('rb') as fd:
+        with path.open('rb') as fd:
             if file_is_binary_database(fd):
-                super().__init__(parse_binary(fd))
-                self._export = write_binary
-                return
+                return _BinaryDatabase(path, fd)
 
         # Read the path as a CSV file.
-        _check_that_file_is_csv_database(self.path)
-        with self.path.open('r', newline='', encoding='utf-8') as file:
-            super().__init__(parse_csv(file))
-            self._export = write_csv
+        _check_that_file_is_csv_database(path)
+        with path.open('r', newline='', encoding='utf-8') as csv_fd:
+            return _CSVDatabase(path, csv_fd)
 
-    def write_to_file(self, path: Optional[Union[Path, str]] = None) -> None:
-        """Exports in the original format to the original or provided path."""
-        with open(self.path if path is None else path, 'wb') as fd:
-            self._export(self, fd)
+    @abstractmethod
+    def write_to_file(self, *, rewrite: bool = False) -> None:
+        """Exports in the original format to the original path."""
+
+    @abstractmethod
+    def add_and_discard_temporary(self,
+                                  entries: Iterable[TokenizedStringEntry],
+                                  commit: str) -> None:
+        """Discards and adds entries to export in the original format.
+
+        Adds entries after removing temporary entries from the Database
+        to exclusively write re-occurring entries into memory and disk.
+        """
+
+
+class _BinaryDatabase(DatabaseFile):
+    def __init__(self, path: Path, fd: BinaryIO) -> None:
+        super().__init__(path, parse_binary(fd))
+
+    def write_to_file(self, *, rewrite: bool = False) -> None:
+        """Exports in the binary format to the original path."""
+        del rewrite  # Binary databases are always rewritten
+        with self.path.open('wb') as fd:
+            write_binary(self, fd)
+
+    def add_and_discard_temporary(self,
+                                  entries: Iterable[TokenizedStringEntry],
+                                  commit: str) -> None:
+        # TODO(b/241471465): Implement adding new tokens and removing
+        # temporary entries for binary databases.
+        raise NotImplementedError('--discard-temporary is currently only '
+                                  'supported for directory databases')
+
+
+class _CSVDatabase(DatabaseFile):
+    def __init__(self, path: Path, fd: TextIO) -> None:
+        super().__init__(path, parse_csv(fd))
+
+    def write_to_file(self, *, rewrite: bool = False) -> None:
+        """Exports in the CSV format to the original path."""
+        del rewrite  # CSV databases are always rewritten
+        with self.path.open('wb') as fd:
+            write_csv(self, fd)
+
+    def add_and_discard_temporary(self,
+                                  entries: Iterable[TokenizedStringEntry],
+                                  commit: str) -> None:
+        # TODO(b/241471465): Implement adding new tokens and removing
+        # temporary entries for CSV databases.
+        raise NotImplementedError('--discard-temporary is currently only '
+                                  'supported for directory databases')
+
+
+# The suffix used for CSV files in a directory database.
+DIR_DB_SUFFIX = '.pw_tokenizer.csv'
+_DIR_DB_GLOB = '*' + DIR_DB_SUFFIX
+
+
+def _parse_directory(directory: Path) -> Iterable[TokenizedStringEntry]:
+    """Parses TokenizedStringEntries tokenizer CSV files in the directory."""
+    for path in directory.glob(_DIR_DB_GLOB):
+        with path.open() as fd:
+            yield from parse_csv(fd)
+
+
+def _most_recently_modified_file(paths: Iterable[Path]) -> Path:
+    return max(paths, key=lambda path: path.stat().st_mtime)
+
+
+class _DirectoryDatabase(DatabaseFile):
+    def __init__(self, directory: Path) -> None:
+        super().__init__(directory, _parse_directory(directory))
+
+    def write_to_file(self, *, rewrite: bool = False) -> None:
+        """Creates a new CSV file in the directory with any new tokens."""
+        if rewrite:
+            # Write the entire database to a new CSV file
+            new_file = self._create_filename()
+            with new_file.open('wb') as fd:
+                write_csv(self, fd)
+
+            # Delete all CSV files except for the new CSV with everything.
+            for csv_file in self.path.glob(_DIR_DB_GLOB):
+                if csv_file != new_file:
+                    csv_file.unlink()
+        else:
+            # Reread the tokens from disk and write only the new entries to CSV.
+            current_tokens = Database(_parse_directory(self.path))
+            new_entries = self.difference(current_tokens)
+            if new_entries:
+                with self._create_filename().open('wb') as fd:
+                    write_csv(new_entries, fd)
+
+    def _git_paths(self, commands: List) -> List[Path]:
+        """Returns a list of files from a Git command, filtered to matc."""
+        try:
+            output = subprocess.run(['git', *commands, _DIR_DB_GLOB],
+                                    capture_output=True,
+                                    check=True,
+                                    cwd=self.path,
+                                    text=True).stdout.strip()
+            return [self.path / repo_path for repo_path in output.splitlines()]
+        except subprocess.CalledProcessError:
+            return []
+
+    def _find_latest_csv(self, commit: str) -> Path:
+        """Finds or creates a CSV to which to write new entries.
+
+        - Check for untracked CSVs. Use the most recently modified file, if any.
+        - Check for CSVs added in HEAD, if HEAD is not an ancestor of commit.
+          Use the most recently modified file, if any.
+        - If no untracked or committed files were found, create a new file.
+        """
+
+        # Prioritize untracked files in the directory database.
+        untracked_changes = self._git_paths(
+            ['ls-files', '--others', '--exclude-standard'])
+        if untracked_changes:
+            return _most_recently_modified_file(untracked_changes)
+
+        # Check if HEAD is an ancestor of the base commit. This checks whether
+        # the top commit has been merged or not. If it has been merged, create a
+        # new CSV to use. Otherwise, check if a CSV was added in the commit.
+        head_is_not_merged = subprocess.run(
+            ['git', 'merge-base', '--is-ancestor', 'HEAD', commit],
+            cwd=self.path,
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.DEVNULL).returncode != 0
+
+        if head_is_not_merged:
+            # Find CSVs added in the top commit.
+            csvs_from_top_commit = self._git_paths([
+                'diff', '--name-only', '--diff-filter=A', '--relative', 'HEAD~'
+            ])
+
+            if csvs_from_top_commit:
+                return _most_recently_modified_file(csvs_from_top_commit)
+
+        return self._create_filename()
+
+    def _create_filename(self) -> Path:
+        """Generates a unique filename not in the directory."""
+        # Tracked and untracked files do not exist in the repo.
+        while (file := self.path / f'{uuid4().hex}{DIR_DB_SUFFIX}').exists():
+            pass
+        return file
+
+    def add_and_discard_temporary(self,
+                                  entries: Iterable[TokenizedStringEntry],
+                                  commit: str) -> None:
+        """Adds new entries and discards temporary entries on disk.
+
+        - Find the latest CSV in the directory database or create a new one.
+        - Delete entries in the latest CSV that are not in the entries passed to
+          this function.
+        - Add the new entries to this database.
+        - Overwrite the latest CSV with only the newly added entries.
+        """
+        # Find entries not currently in the database.
+        added = Database(entries)
+        new_entries = added.difference(self)
+
+        csv_path = self._find_latest_csv(commit)
+        if csv_path.exists():
+            # Loading the CSV as a DatabaseFile.
+            csv_db = DatabaseFile.load(csv_path)
+
+            # Delete entries added in the CSV, but not added in this function.
+            for key in (e.key() for e in csv_db.difference(added).entries()):
+                del self._database[key]
+                del csv_db._database[key]  # pylint: disable=protected-access
+
+            csv_db.add(new_entries.entries())
+            csv_db.write_to_file()
+        elif new_entries:  # If the CSV does not exist, write all new tokens.
+            with csv_path.open('wb') as fd:
+                write_csv(new_entries, fd)
+
+        self.add(new_entries.entries())
