@@ -23,7 +23,11 @@ from pw_status import Status
 
 from pw_transfer.transfer import (ProgressCallback, ReadTransfer, Transfer,
                                   WriteTransfer)
-from pw_transfer.transfer_pb2 import Chunk
+try:
+    from pw_transfer import transfer_pb2
+except ImportError:
+    # For the bazel build, which puts generated protos in a different location.
+    from pigweed.pw_transfer import transfer_pb2  # type: ignore
 
 _LOG = logging.getLogger(__package__)
 
@@ -70,6 +74,8 @@ class Manager:  # pylint: disable=too-many-instance-attributes
         self._write_stream: Optional[BidirectionalStreamingCall] = None
 
         self._loop = asyncio.new_event_loop()
+        # Set the event loop for the current thread.
+        asyncio.set_event_loop(self._loop)
 
         # Queues are used for communication between the Manager context and the
         # dedicated asyncio transfer thread.
@@ -156,11 +162,11 @@ class Manager:  # pylint: disable=too-many-instance-attributes
         if not transfer.status.ok():
             raise Error(transfer.id, transfer.status)
 
-    def _send_read_chunk(self, chunk: Chunk) -> None:
+    def _send_read_chunk(self, chunk: transfer_pb2.Chunk) -> None:
         assert self._read_stream is not None
         self._read_stream.send(chunk)
 
-    def _send_write_chunk(self, chunk: Chunk) -> None:
+    def _send_write_chunk(self, chunk: transfer_pb2.Chunk) -> None:
         assert self._write_stream is not None
         self._write_stream.send(chunk)
 
@@ -216,7 +222,8 @@ class Manager:  # pylint: disable=too-many-instance-attributes
         self._loop.stop()
 
     @staticmethod
-    async def _handle_chunk(transfers: _TransferDict, chunk: Chunk) -> None:
+    async def _handle_chunk(transfers: _TransferDict,
+                            chunk: transfer_pb2.Chunk) -> None:
         """Processes an incoming chunk from a stream.
 
         The chunk is dispatched to an active transfer based on its ID. If the
@@ -225,11 +232,11 @@ class Manager:  # pylint: disable=too-many-instance-attributes
         """
 
         try:
-            transfer = transfers[chunk.session_id]
+            transfer = transfers[chunk.transfer_id]
         except KeyError:
             _LOG.error(
                 'TransferManager received chunk for unknown transfer %d',
-                chunk.session_id)
+                chunk.transfer_id)
             # TODO(frolv): What should be done here, if anything?
             return
 
