@@ -13,4 +13,5 @@
 # the License.
 """Utilities for running unit tests over Pigweed RPC."""
 
-from pw_unit_test.rpc import run_tests, EventHandler, TestCase
+from pw_unit_test.rpc import (run_tests, EventHandler, TestCase,
+                              TestExpectation, TestCaseResult)
